@@ -18,30 +18,35 @@ from moto.awslambda import models as lambda_models
 from moto.apigateway import models as apigw_models
 from moto.cloudwatch import models as cw_models
 from moto.cloudformation import parsing, responses
+from moto.cloudformation import utils as cloudformation_utils
+from moto.cloudformation import models as cloudformation_models
 from boto.cloudformation.stack import Output
-from moto.cloudformation.models import FakeStack, cloudformation_backends
+from moto.cloudformation.models import FakeStack, CloudFormationBackend, cloudformation_backends
 from moto.cloudformation.exceptions import ValidationError, UnformattedGetAttTemplateException
 from localstack import config
-from localstack.constants import DEFAULT_PORT_CLOUDFORMATION_BACKEND, TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
+from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
-from localstack.utils.common import FuncThread, short_uid, recurse_object, clone, json_safe
+from localstack.utils.common import (
+    FuncThread, short_uid, recurse_object, clone, json_safe, md5, canonical_json, get_free_tcp_port)
 from localstack.stepfunctions import models as sfn_models
 from localstack.services.infra import (
-    get_service_protocol, start_proxy_for_service, do_run, canonicalize_api_names)
+    get_service_protocol, start_proxy_for_service, do_run, canonicalize_api_names
+)
 from localstack.utils.bootstrap import setup_logging
 from localstack.utils.cloudformation import template_deployer
 from localstack.services.cloudformation import service_models
-from localstack.services.awslambda.lambda_api import BUCKET_MARKER_LOCAL
 
 LOG = logging.getLogger(__name__)
 
+MOTO_CFN_ACCOUNT_ID = '123456789'
+
 # Maps (stack_name,resource_logical_id) -> Bool to indicate which resources are currently being updated
 CURRENTLY_UPDATING_RESOURCES = {}
 
 # whether to start the API in a separate process
 RUN_SERVER_IN_PROCESS = False
 
-# maxiumum depth of the resource dependency tree
+# maximum depth of the resource dependency tree
 MAX_DEPENDENCY_DEPTH = 40
 
 # map of additional model classes
@@ -54,14 +59,22 @@ MODEL_MAP = {
     'AWS::ApiGateway::Resource': apigw_models.Resource,
     'AWS::ApiGateway::RestApi': apigw_models.RestAPI,
     'AWS::StepFunctions::StateMachine': sfn_models.StateMachine,
-    'AWS::CloudFormation::Stack': service_models.CloudFormationStack
+    'AWS::CloudFormation::Stack': service_models.CloudFormationStack,
+    'AWS::SSM::Parameter': service_models.SSMParameter,
+    'AWS::Logs::LogGroup': service_models.LogsLogGroup,
+    'AWS::KinesisFirehose::DeliveryStream': service_models.FirehoseDeliveryStream,
+    'AWS::SecretsManager::Secret': service_models.SecretsManagerSecret,
+    'AWS::Elasticsearch::Domain': service_models.ElasticsearchDomain,
+    'AWS::Events::Rule': service_models.EventsRule,
+    'AWS::S3::BucketPolicy': service_models.S3BucketPolicy
 }
 
 
 def start_cloudformation(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_CLOUDFORMATION
-    backend_port = DEFAULT_PORT_CLOUDFORMATION_BACKEND
-    print('Starting mock CloudFormation (%s port %s)...' % (get_service_protocol(), port))
+    print('Starting mock CloudFormation service in %s ports %s (recommended) and %s (deprecated)...' % (
+        get_service_protocol(), config.EDGE_PORT, port))
+    backend_port = get_free_tcp_port()
     start_proxy_for_service('cloudformation', port, backend_port, update_listener)
     if RUN_SERVER_IN_PROCESS:
         cmd = 'python "%s" cloudformation -p %s -H 0.0.0.0' % (__file__, backend_port)
@@ -132,39 +145,56 @@ def update_physical_resource_id(resource):
         if isinstance(resource, lambda_models.LambdaFunction):
             func_arn = aws_stack.lambda_function_arn(resource.function_name)
             resource.function_arn = resource.physical_resource_id = func_arn
+
         elif isinstance(resource, sfn_models.StateMachine):
             sm_arn = aws_stack.state_machine_arn(resource.name)
             resource.physical_resource_id = sm_arn
+
         elif isinstance(resource, service_models.StepFunctionsActivity):
             act_arn = aws_stack.stepfunctions_activity_arn(resource.params.get('Name'))
             resource.physical_resource_id = act_arn
+
         elif isinstance(resource, kinesis_models.Stream):
             resource.physical_resource_id = resource.stream_name
+
+        elif isinstance(resource, service_models.LogsLogGroup):
+            resource.physical_resource_id = resource.params.get('LogGroupName')
+
+        elif isinstance(resource, service_models.FirehoseDeliveryStream):
+            resource.physical_resource_id = resource.params.get('DeliveryStreamName')
+
+        elif isinstance(resource, service_models.SecretsManagerSecret):
+            resource.physical_resource_id = resource.params.get('Name')
+
+        elif isinstance(resource, service_models.EventsRule):
+            resource.physical_resource_id = resource.params.get('Name')
+
+        elif isinstance(resource, service_models.ElasticsearchDomain):
+            resource.physical_resource_id = resource.params.get('DomainName')
+
         else:
             LOG.warning('Unable to determine physical_resource_id for resource %s' % type(resource))
 
 
+def update_resource_name(resource, resource_json):
+    """ Some resources require minor fixes in their CF resource definition
+        before we can pass them on to deployment. """
+    props = resource_json['Properties'] = resource_json.get('Properties') or {}
+    if isinstance(resource, sfn_models.StateMachine) and not props.get('StateMachineName'):
+        props['StateMachineName'] = resource.name
+
+
 def apply_patches():
     """ Apply patches to make LocalStack seamlessly interact with the moto backend.
         TODO: Eventually, these patches should be contributed to the upstream repo! """
 
     # add model mappings to moto
-
     parsing.MODEL_MAP.update(MODEL_MAP)
 
-    # Patch S3Backend.get_key method in moto to use S3 API from LocalStack
-
-    def get_key(self, bucket_name, key_name, version_id=None):
-        s3_client = aws_stack.connect_to_service('s3')
-        value = b''
-        if bucket_name != BUCKET_MARKER_LOCAL:
-            value = s3_client.get_object(Bucket=bucket_name, Key=key_name)['Body'].read()
-        return s3_models.FakeKey(name=key_name, value=value)
-
-    s3_models.S3Backend.get_key = get_key
+    # fix account ID
+    parsing.ACCOUNT_ID = TEST_AWS_ACCOUNT_ID
 
     # Patch clean_json in moto
-
     def clean_json(resource_json, resources_map):
         result = clean_json_orig(resource_json, resources_map)
         if isinstance(result, BaseModel):
@@ -180,11 +210,11 @@ def apply_patches():
     parsing.clean_json = clean_json
 
     # Patch parse_and_create_resource method in moto to deploy resources in LocalStack
-
     def parse_and_create_resource(logical_id, resource_json, resources_map, region_name, force_create=False):
         try:
-            return _parse_and_create_resource(logical_id, resource_json,
-                resources_map, region_name, force_create=force_create)
+            return _parse_and_create_resource(
+                logical_id, resource_json, resources_map, region_name, force_create=force_create
+            )
         except Exception as e:
             LOG.error('Unable to parse and create resource "%s": %s %s' %
                       (logical_id, e, traceback.format_exc()))
@@ -192,8 +222,7 @@ def apply_patches():
 
     def parse_and_update_resource(logical_id, resource_json, resources_map, region_name):
         try:
-            return _parse_and_create_resource(logical_id,
-                resource_json, resources_map, region_name, update=True)
+            return _parse_and_create_resource(logical_id, resource_json, resources_map, region_name, update=True)
         except Exception as e:
             LOG.error('Unable to parse and update resource "%s": %s %s' %
                       (logical_id, e, traceback.format_exc()))
@@ -203,6 +232,7 @@ def apply_patches():
             update=False, force_create=False):
         stack_name = resources_map.get('AWS::StackName')
         resource_hash_key = (stack_name, logical_id)
+        props = resource_json['Properties'] = resource_json.get('Properties') or {}
 
         # If the current stack is being updated, avoid infinite recursion
         updating = CURRENTLY_UPDATING_RESOURCES.get(resource_hash_key)
@@ -216,40 +246,60 @@ def apply_patches():
             return None
         _, resource_json, _ = resource_tuple
 
-        # add some missing default props which otherwise cause deployments to fail
-        props = resource_json['Properties'] = resource_json.get('Properties') or {}
-        if resource_json['Type'] == 'AWS::Lambda::EventSourceMapping' and not props.get('StartingPosition'):
-            props['StartingPosition'] = 'LATEST'
+        def add_default_props(resource_props):
+            """ apply some fixes which otherwise cause deployments to fail """
+            res_type = resource_props['Type']
+            props = resource_props.get('Properties', {})
+            if res_type == 'AWS::Lambda::EventSourceMapping' and not props.get('StartingPosition'):
+                props['StartingPosition'] = 'LATEST'
+            # generate default names for certain resource types
+            default_attrs = (('AWS::IAM::Role', 'RoleName'), ('AWS::Events::Rule', 'Name'))
+            for entry in default_attrs:
+                if res_type == entry[0] and not props.get(entry[1]):
+                    props[entry[1]] = 'cf-%s-%s' % (stack_name, md5(canonical_json(props)))
+
+        # add some fixes and default props which otherwise cause deployments to fail
+        add_default_props(resource_json)
+        for resource in resources_map._resource_json_map.values():
+            add_default_props(resource)
 
         # check if this resource already exists in the resource map
         resource = resources_map._parsed_resources.get(logical_id)
         if resource and not update and not force_create:
             return resource
 
-        # check whether this resource needs to be deployed
-        resource_map_new = dict(resources_map._resource_json_map)
-        resource_map_new[logical_id] = resource_json
-        should_be_created = template_deployer.should_be_deployed(logical_id, resource_map_new, stack_name)
-
         # fix resource ARNs, make sure to convert account IDs 000000000000 to 123456789012
         resource_json_arns_fixed = clone(json_safe(convert_objs_to_ids(resource_json)))
         set_moto_account_ids(resource_json_arns_fixed)
 
         # create resource definition and store CloudFormation metadata in moto
+        moto_create_error = None
         if (resource or update) and not force_create:
-            parse_and_update_resource_orig(logical_id,
-                resource_json_arns_fixed, resources_map, region_name)
+            parse_and_update_resource_orig(logical_id, resource_json_arns_fixed, resources_map, region_name)
         elif not resource:
             try:
-                resource = parse_and_create_resource_orig(logical_id,
-                    resource_json_arns_fixed, resources_map, region_name)
+                resource = parse_and_create_resource_orig(
+                    logical_id, resource_json_arns_fixed, resources_map, region_name
+                )
+                if not resource:
+                    # this can happen if the resource has an associated Condition which evaluates to false
+                    return resource
                 resource.logical_id = logical_id
             except Exception as e:
-                if should_be_created:
-                    raise
-                else:
-                    LOG.info('Error on moto CF resource creation. Ignoring, as should_be_created=%s: %s' %
-                             (should_be_created, e))
+                moto_create_error = e
+
+        # check whether this resource needs to be deployed
+        resource_map_new = dict(resources_map._resource_json_map)
+        resource_map_new[logical_id] = resource_json
+        should_be_created = template_deployer.should_be_deployed(logical_id, resource_map_new, stack_name)
+
+        # check for moto creation errors and raise an exception if needed
+        if moto_create_error:
+            if should_be_created:
+                raise moto_create_error
+            else:
+                LOG.info('Error on moto CF resource creation. Ignoring, as should_be_created=%s: %s' %
+                         (should_be_created, moto_create_error))
 
         # Fix for moto which sometimes hard-codes region name as 'us-east-1'
         if hasattr(resource, 'region_name') and resource.region_name != region_name:
@@ -263,7 +313,8 @@ def apply_patches():
             is_updateable = template_deployer.is_updateable(logical_id, resource_map_new, stack_name)
             if not update or not is_updateable:
                 all_satisfied = template_deployer.all_resource_dependencies_satisfied(
-                    logical_id, resource_map_new, stack_name)
+                    logical_id, resource_map_new, stack_name
+                )
                 if not all_satisfied:
                     LOG.info('Resource %s cannot be deployed, found unsatisfied dependencies. %s' % (
                         logical_id, resource_json))
@@ -309,7 +360,8 @@ def apply_patches():
             LOG.debug('Updating resource id: %s - %s, %s - %s' % (existing_id, new_res_id, resource, resource_json))
             if new_res_id:
                 LOG.info('Updating resource ID from %s to %s (%s)' % (existing_id, new_res_id, region_name))
-                update_resource_id(resource, new_res_id, props, region_name)
+                update_resource_id(resource, new_res_id, props,
+                    region_name, stack_name, resources_map._resource_json_map)
             else:
                 LOG.warning('Unable to extract id for resource %s: %s' % (logical_id, result))
 
@@ -318,14 +370,7 @@ def apply_patches():
 
         return resource
 
-    def update_resource_name(resource, resource_json):
-        """ Some resources require minor fixes in their CF resource definition
-            before we can pass them on to deployment. """
-        props = resource_json['Properties'] = resource_json.get('Properties') or {}
-        if isinstance(resource, sfn_models.StateMachine) and not props.get('StateMachineName'):
-            props['StateMachineName'] = resource.name
-
-    def update_resource_id(resource, new_id, props, region_name):
+    def update_resource_id(resource, new_id, props, region_name, stack_name, resource_map):
         """ Update and fix the ID(s) of the given resource. """
 
         # NOTE: this is a bit of a hack, which is required because
@@ -360,11 +405,13 @@ def apply_patches():
             resource.id = new_id
         elif isinstance(resource, apigw_models.Resource):
             api_id = props['RestApiId']
+            api_id = template_deployer.resolve_refs_recursively(stack_name, api_id, resource_map)
             backend.apis[api_id].resources.pop(resource.id, None)
             backend.apis[api_id].resources[new_id] = resource
             resource.id = new_id
         elif isinstance(resource, apigw_models.Deployment):
             api_id = props['RestApiId']
+            api_id = template_deployer.resolve_refs_recursively(stack_name, api_id, resource_map)
             backend.apis[api_id].deployments.pop(resource['id'], None)
             backend.apis[api_id].deployments[new_id] = resource
             resource['id'] = new_id
@@ -376,8 +423,7 @@ def apply_patches():
     parse_and_update_resource_orig = parsing.parse_and_update_resource
     parsing.parse_and_update_resource = parse_and_update_resource
 
-    # Patch CloudFormation parse_output(..) method to fix a bug in moto
-
+    # patch CloudFormation parse_output(..) method to fix a bug in moto
     def parse_output(output_logical_id, output_json, resources_map):
         try:
             result = parse_output_orig(output_logical_id, output_json, resources_map)
@@ -395,7 +441,6 @@ def apply_patches():
     parsing.parse_output = parse_output
 
     # Make sure the export name is returned for stack outputs
-
     if '<ExportName>' not in responses.DESCRIBE_STACKS_TEMPLATE:
         find = '</OutputValue>'
         replace = """</OutputValue>
@@ -406,7 +451,6 @@ def apply_patches():
         responses.DESCRIBE_STACKS_TEMPLATE = responses.DESCRIBE_STACKS_TEMPLATE.replace(find, replace)
 
     # Patch CloudFormationBackend.update_stack method in moto
-
     def make_cf_update_stack(cf_backend):
         cf_update_stack_orig = cf_backend.update_stack
 
@@ -423,20 +467,26 @@ def apply_patches():
         cf_backend.update_stack = make_cf_update_stack(cf_backend)
 
     # Patch DynamoDB get_cfn_attribute(..) method in moto
-
     def DynamoDB_Table_get_cfn_attribute(self, attribute_name):
         try:
-            return DynamoDB_Table_get_cfn_attribute_orig(self, attribute_name)
+            return ddb_table_get_cfn_attribute_orig(self, attribute_name)
         except Exception:
             if attribute_name == 'Arn':
                 return aws_stack.dynamodb_table_arn(table_name=self.name)
             raise
 
-    DynamoDB_Table_get_cfn_attribute_orig = dynamodb_models.Table.get_cfn_attribute
+    ddb_table_get_cfn_attribute_orig = dynamodb_models.Table.get_cfn_attribute
     dynamodb_models.Table.get_cfn_attribute = DynamoDB_Table_get_cfn_attribute
 
-    # Patch DynamoDB get_cfn_attribute(..) method in moto
+    # Patch generate_stack_id(..) method in moto
+    def generate_stack_id(stack_name, region=None, **kwargs):
+        region = region or aws_stack.get_region()
+        return generate_stack_id_orig(stack_name, region=region, **kwargs)
 
+    generate_stack_id_orig = cloudformation_utils.generate_stack_id
+    cloudformation_utils.generate_stack_id = cloudformation_models.generate_stack_id = generate_stack_id
+
+    # Patch DynamoDB get_cfn_attribute(..) method in moto
     def DynamoDB2_Table_get_cfn_attribute(self, attribute_name):
         if attribute_name == 'Arn':
             return aws_stack.dynamodb_table_arn(table_name=self.name)
@@ -449,7 +499,6 @@ def apply_patches():
     dynamodb2_models.Table.get_cfn_attribute = DynamoDB2_Table_get_cfn_attribute
 
     # Patch SQS get_cfn_attribute(..) method in moto
-
     def SQS_Queue_get_cfn_attribute(self, attribute_name):
         if attribute_name in ['Arn', 'QueueArn']:
             return aws_stack.sqs_queue_arn(queue_name=self.name)
@@ -459,7 +508,6 @@ def apply_patches():
     sqs_models.Queue.get_cfn_attribute = SQS_Queue_get_cfn_attribute
 
     # Patch S3 Bucket get_cfn_attribute(..) method in moto
-
     def S3_Bucket_get_cfn_attribute(self, attribute_name):
         if attribute_name in ['Arn']:
             return aws_stack.s3_bucket_arn(self.name)
@@ -469,7 +517,6 @@ def apply_patches():
     s3_models.FakeBucket.get_cfn_attribute = S3_Bucket_get_cfn_attribute
 
     # Patch SQS physical_resource_id(..) method in moto
-
     @property
     def SQS_Queue_physical_resource_id(self):
         result = SQS_Queue_physical_resource_id_orig.fget(self)
@@ -482,7 +529,6 @@ def apply_patches():
     sqs_models.Queue.physical_resource_id = SQS_Queue_physical_resource_id
 
     # Patch LogGroup get_cfn_attribute(..) method in moto
-
     def LogGroup_get_cfn_attribute(self, attribute_name):
         try:
             return LogGroup_get_cfn_attribute_orig(self, attribute_name)
@@ -495,7 +541,6 @@ def apply_patches():
     cw_models.LogGroup.get_cfn_attribute = LogGroup_get_cfn_attribute
 
     # Patch Lambda get_cfn_attribute(..) method in moto
-
     def Lambda_Function_get_cfn_attribute(self, attribute_name):
         try:
             if attribute_name == 'Arn':
@@ -510,7 +555,6 @@ def apply_patches():
     lambda_models.LambdaFunction.get_cfn_attribute = Lambda_Function_get_cfn_attribute
 
     # Patch DynamoDB get_cfn_attribute(..) method in moto
-
     def DynamoDB_Table_get_cfn_attribute(self, attribute_name):
         try:
             if attribute_name == 'StreamArn':
@@ -525,7 +569,6 @@ def apply_patches():
     dynamodb_models.Table.get_cfn_attribute = DynamoDB_Table_get_cfn_attribute
 
     # Patch IAM get_cfn_attribute(..) method in moto
-
     def IAM_Role_get_cfn_attribute(self, attribute_name):
         try:
             return IAM_Role_get_cfn_attribute_orig(self, attribute_name)
@@ -537,8 +580,15 @@ def apply_patches():
     IAM_Role_get_cfn_attribute_orig = iam_models.Role.get_cfn_attribute
     iam_models.Role.get_cfn_attribute = IAM_Role_get_cfn_attribute
 
-    # Patch SNS Topic get_cfn_attribute(..) method in moto
+    # Patch IAM Role model
+    # https://github.com/localstack/localstack/issues/925
+    @property
+    def IAM_Role_physical_resource_id(self):
+        return self.name
 
+    iam_models.Role.physical_resource_id = IAM_Role_physical_resource_id
+
+    # Patch SNS Topic get_cfn_attribute(..) method in moto
     def SNS_Topic_get_cfn_attribute(self, attribute_name):
         result = SNS_Topic_get_cfn_attribute_orig(self, attribute_name)
         if attribute_name.lower() in ['arn', 'topicarn']:
@@ -548,8 +598,43 @@ def apply_patches():
     SNS_Topic_get_cfn_attribute_orig = sns_models.Topic.get_cfn_attribute
     sns_models.Topic.get_cfn_attribute = SNS_Topic_get_cfn_attribute
 
-    # Patch LambdaFunction create_from_cloudformation_json(..) method in moto
+    # Patch create_from_cloudformation_json(..)
+    # #2568 Cloudformation create-stack for SNS with yaml causes TypeError
 
+    SNS_Topic_create_from_cloudformation_json_orig = sns_models.Topic.create_from_cloudformation_json
+
+    def SNS_Topic_create_from_cloudformation_json(resource_name, cloudformation_json, region_name):
+        properties = cloudformation_json['Properties']
+        if properties.get('Subscription'):
+            properties['Subscription'] = [subscription for subscription in properties['Subscription'] if subscription]
+
+        return SNS_Topic_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+
+    sns_models.Topic.create_from_cloudformation_json = SNS_Topic_create_from_cloudformation_json
+
+    # Patch ES get_cfn_attribute(..) method
+    def ES_get_cfn_attribute(self, attribute_name):
+        if attribute_name in ['Arn', 'DomainArn']:
+            return aws_stack.es_domain_arn(self.params.get('DomainName'))
+        if attribute_name == 'DomainEndpoint':
+            if not hasattr(self, '_domain_endpoint'):
+                es_details = aws_stack.connect_to_service('es').describe_elasticsearch_domain(
+                    DomainName=self.params.get('DomainName'))
+                self._domain_endpoint = es_details['DomainStatus']['Endpoint']
+            return self._domain_endpoint
+        raise UnformattedGetAttTemplateException()
+
+    service_models.ElasticsearchDomain.get_cfn_attribute = ES_get_cfn_attribute
+
+    # Patch Firehose get_cfn_attribute(..) method
+    def Firehose_get_cfn_attribute(self, attribute_name):
+        if attribute_name == 'Arn':
+            return aws_stack.firehose_stream_arn(self.params.get('DeliveryStreamName'))
+        raise UnformattedGetAttTemplateException()
+
+    service_models.FirehoseDeliveryStream.get_cfn_attribute = Firehose_get_cfn_attribute
+
+    # Patch LambdaFunction create_from_cloudformation_json(..) method in moto
     @classmethod
     def Lambda_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         resource_name = cloudformation_json.get('Properties', {}).get('FunctionName') or resource_name
@@ -559,7 +644,6 @@ def apply_patches():
     lambda_models.LambdaFunction.create_from_cloudformation_json = Lambda_create_from_cloudformation_json
 
     # Patch EventSourceMapping create_from_cloudformation_json(..) method in moto
-
     @classmethod
     def Mapping_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         props = cloudformation_json.get('Properties', {})
@@ -572,7 +656,6 @@ def apply_patches():
     lambda_models.EventSourceMapping.create_from_cloudformation_json = Mapping_create_from_cloudformation_json
 
     # Patch LambdaFunction update_from_cloudformation_json(..) method in moto
-
     @classmethod
     def Lambda_update_from_cloudformation_json(cls,
             original_resource, new_resource_name, cloudformation_json, region_name):
@@ -582,10 +665,20 @@ def apply_patches():
     if not hasattr(lambda_models.LambdaFunction, 'update_from_cloudformation_json'):
         lambda_models.LambdaFunction.update_from_cloudformation_json = Lambda_update_from_cloudformation_json
 
-    # patch ApiGateway Deployment
+    # Patch Role update_from_cloudformation_json(..) method
+    @classmethod
+    def Role_update_from_cloudformation_json(cls,
+            original_resource, new_resource_name, cloudformation_json, region_name):
+        props = cloudformation_json.get('Properties', {})
+        original_resource.name = props.get('RoleName') or original_resource.name
+        original_resource.assume_role_policy_document = props.get('AssumeRolePolicyDocument')
+        return original_resource
 
-    def depl_delete_from_cloudformation_json(
-            resource_name, resource_json, region_name):
+    if not hasattr(iam_models.Role, 'update_from_cloudformation_json'):
+        iam_models.Role.update_from_cloudformation_json = Role_update_from_cloudformation_json
+
+    # patch ApiGateway Deployment
+    def depl_delete_from_cloudformation_json(resource_name, resource_json, region_name):
         properties = resource_json['Properties']
         LOG.info('TODO: apigateway.Deployment.delete_from_cloudformation_json %s' % properties)
 
@@ -593,9 +686,7 @@ def apply_patches():
         apigw_models.Deployment.delete_from_cloudformation_json = depl_delete_from_cloudformation_json
 
     # patch Lambda Version
-
-    def vers_delete_from_cloudformation_json(
-            resource_name, resource_json, region_name):
+    def vers_delete_from_cloudformation_json(resource_name, resource_json, region_name):
         properties = resource_json['Properties']
         LOG.info('TODO: apigateway.Deployment.delete_from_cloudformation_json %s' % properties)
 
@@ -603,7 +694,6 @@ def apply_patches():
         lambda_models.LambdaVersion.delete_from_cloudformation_json = vers_delete_from_cloudformation_json
 
     # add CloudFormation types
-
     @classmethod
     def RestAPI_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         props = cloudformation_json['Properties']
@@ -662,7 +752,6 @@ def apply_patches():
     # TODO: add support for AWS::ApiGateway::Model, AWS::ApiGateway::RequestValidator, ...
 
     # fix AttributeError in moto's CloudFormation describe_stack_resource
-
     def describe_stack_resource(self):
         stack_name = self._get_param('StackName')
         stack = self.cloudformation_backend.get_stack(stack_name)
@@ -692,7 +781,6 @@ def apply_patches():
     responses.CloudFormationResponse.describe_stack_resource = describe_stack_resource
 
     # fix moto's describe_stack_events jinja2.exceptions.UndefinedError
-
     def cf_describe_stack_events(self):
         stack_name = self._get_param('StackName')
         backend = self.cloudformation_backend
@@ -709,31 +797,27 @@ def apply_patches():
     responses.CloudFormationResponse.describe_stack_events = cf_describe_stack_events
 
     # fix Lambda regions in moto - see https://github.com/localstack/localstack/issues/1961
-
     for region in boto3.session.Session().get_available_regions('lambda'):
         if region not in lambda_models.lambda_backends:
             lambda_models.lambda_backends[region] = lambda_models.LambdaBackend(region)
 
     # patch FakeStack.initialize_resources
-
-    def initialize_resources(self):
+    def run_dependencies_deployment_loop(stack, action):
         def set_status(status):
-            self._add_stack_event(status)
-            self.status = status
-
-        self.resource_map.create()
-        self.output_map.create()
+            stack._add_stack_event(status)
+            stack.status = status
 
         def run_loop(*args):
             # NOTE: We're adding this additional loop, as it seems that in some cases moto
             #   does not consider resource dependencies (e.g., if a "DependsOn" resource property
             #   is defined). This loop allows us to incrementally resolve such dependencies.
-            resource_map = self.resource_map
+            resource_map = stack.resource_map
             unresolved = {}
             for i in range(MAX_DEPENDENCY_DEPTH):
+                LOG.debug('Running CloudFormation stack deployment loop iteration %s' % (i + 1))
                 unresolved = getattr(resource_map, '_unresolved_resources', {})
                 if not unresolved:
-                    set_status('CREATE_COMPLETE')
+                    set_status('%s_COMPLETE' % action)
                     return resource_map
                 resource_map._unresolved_resources = {}
                 for resource_id, resource_details in unresolved.items():
@@ -743,14 +827,27 @@ def apply_patches():
                     # looks like no more resources can be resolved -> bail
                     LOG.warning('Unresolvable dependencies, there may be undeployed stack resources: %s' % unresolved)
                     break
-            set_status('CREATE_FAILED')
+            set_status('%s_FAILED' % action)
             raise Exception('Unable to resolve all CloudFormation resources after traversing ' +
-                'dependency tree (maximum depth %s reached): %s' % (MAX_DEPENDENCY_DEPTH, unresolved.keys()))
+                'dependency tree (maximum depth %s reached): %s' % (MAX_DEPENDENCY_DEPTH, list(unresolved.keys())))
 
         # NOTE: We're running the loop in the background, as it might take some time to complete
         FuncThread(run_loop).start()
 
+    def initialize_resources(self):
+        self.resource_map._template = self.resource_map._template or self.template_dict
+        self.resource_map.load()
+        self.resource_map.create(self.template_dict)
+        self.output_map.create()
+        run_dependencies_deployment_loop(self, 'CREATE')
+
+    def update(self, *args, **kwargs):
+        stack_update_orig(self, *args, **kwargs)
+        run_dependencies_deployment_loop(self, 'UPDATE')
+
     FakeStack.initialize_resources = initialize_resources
+    stack_update_orig = FakeStack.update
+    FakeStack.update = update
 
     # patch Kinesis Stream get_cfn_attribute(..) method in moto
     def Kinesis_Stream_get_cfn_attribute(self, attribute_name):
@@ -761,6 +858,69 @@ def apply_patches():
 
     kinesis_models.Stream.get_cfn_attribute = Kinesis_Stream_get_cfn_attribute
 
+    # patch cloudformation backend create_change_set(..)
+    # #760 cloudformation deploy invalid xml error
+    cloudformation_backend_create_change_set_orig = CloudFormationBackend.create_change_set
+
+    def cloudformation_backend_create_change_set(
+            self,
+            stack_name,
+            change_set_name,
+            template,
+            parameters,
+            region_name,
+            change_set_type,
+            notification_arns=None,
+            tags=None,
+            role_arn=None):
+        change_set_id, _ = cloudformation_backend_create_change_set_orig(
+            self,
+            stack_name,
+            change_set_name,
+            template,
+            parameters,
+            region_name,
+            change_set_type,
+            notification_arns,
+            tags,
+            role_arn
+        )
+        change_set = self.change_sets[change_set_id]
+        change_set.status = 'CREATE_COMPLETE'
+
+        return change_set_id, _
+
+    CloudFormationBackend.create_change_set = cloudformation_backend_create_change_set
+
+    # patch cloudformation backend change_set methods
+    # #2240 - S3 bucket not created since 0.10.8
+    # #2568 - Cloudformation create-stack for SNS with yaml causes TypeError
+    cloudformation_backend_describe_change_set_orig = CloudFormationBackend.describe_change_set
+    cloudformation_backend_execute_change_set_orig = CloudFormationBackend.execute_change_set
+
+    def cloudformation_backend_describe_change_set(self, change_set_name, stack_name=None):
+        change_set_name = change_set_name.replace(TEST_AWS_ACCOUNT_ID, MOTO_CFN_ACCOUNT_ID)
+        return cloudformation_backend_describe_change_set_orig(self, change_set_name, stack_name)
+
+    def cloudformation_backend_execute_change_set(self, change_set_name, stack_name=None):
+        change_set_name = change_set_name.replace(TEST_AWS_ACCOUNT_ID, MOTO_CFN_ACCOUNT_ID)
+        resp = cloudformation_backend_execute_change_set_orig(self, change_set_name, stack_name)
+
+        stack = self.change_sets.get(change_set_name)
+        if not stack:
+            for cs in self.change_sets:
+                if self.change_sets[cs].change_set_name == change_set_name:
+                    stack = self.change_sets[cs]
+
+        stack.output_map = stack._create_output_map()
+        for export in stack.exports:
+            self.exports[export.name] = export
+
+        return resp
+
+    CloudFormationBackend.describe_change_set = cloudformation_backend_describe_change_set
+    CloudFormationBackend.execute_change_set = cloudformation_backend_execute_change_set
+
 
 def inject_stats_endpoint():
     """ Inject a simple /_stats endpoint into the moto server backend Web app. """
@@ -772,7 +932,7 @@ def inject_stats_endpoint():
         all_objects = muppy.get_objects()
         result = summary.summarize(all_objects)
         result = result[0:20]
-        summary = '\n'.join([l for l in summary.format_(result)])
+        summary = '\n'.join([line for line in summary.format_(result)])
         result = '%s\n\n%s' % (summary, json.dumps(result))
         return result, 200, {'content-type': 'text/plain'}
 
